package com.hh.arome.lib_nfc;

import android.content.Context;
import android.util.Log;

import com.donsee.devices.CardReader;
import com.donsee.devices.HexUtil;

/**
 * @Description:
 * @author: Xia WeiHao
 * @date: 2024/6/27
 */
public class DX_NFCCardReader {

    private NFCCardConfig config;
    private NFCCardResaultListener listener;
    private CardReader cardReader;

    private volatile CReaderThread cReaderDataThread;

    private volatile boolean isRuning = false;

    private volatile String oldNFCResetInfo = "-2";
    private volatile long recordTime = 0L;


    public DX_NFCCardReader(Context context, NFCCardConfig config, NFCCardResaultListener listener) {
        this.listener = listener;
        this.config = config;
        init(context);
    }

    private void init(Context context) {
        cardReader = new CardReader(context);
        long result = cardReader.Donsee_Open("/dev/ttysWK0");
        if (result != 0 && listener != null) {
            listener.onError("连接串口设备失败");
            return;
        }
    }

    public void setConfig(NFCCardConfig config) {
        this.config = config;
    }

    private class CReaderThread extends Thread {
        private final Object lock = new Object();
        private volatile boolean pause = false;

        public CReaderThread() {
        }

        void pauseThread() {
            pause = true;
        }

        void resumeThread() {
            pause = false;
            synchronized (lock) {
                lock.notifyAll();
            }
        }

        void onPause() {
            synchronized (lock) {
                try {
                    recordTime = 0L;
                    lock.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        @Override
        public void run() {
            super.run();
            while (isRuning) {
                while (pause) onPause();
                readNFC();
                try {//后面这些N个try是给性能没那么好的cpu预留的  没啥特殊需求就留着
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    cReaderDataThread = null;
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * desc:获取单次卡片信息
     *
     * @author xia weihao
     * @date 2024/6/27
     */
    public String getOnceCardInfo() {
        byte[] uid = new byte[10];
        byte type = (byte) 0x41; //0x41 表示TypeA/M1卡选卡,  0x43表示TypeB卡选卡
        byte[] chResponse = new byte[2048];
        if (cardReader == null) {
            listener.onError("getOnceCardInfo：cardReader = null");
            return "-1";
        }
        long ret = cardReader.Donsee_ICRequest();
        if (ret != 0) {
            listener.onError("TypeA寻卡失败: " + CardReader.getErrorMessage(ret));
            return "-2";
        }
        if (cardReader.Donsee_ICAnticoll(uid) < 0) {
            listener.onError("TypeA防碰撞失败: " + CardReader.getErrorMessage(ret));
            return "-3";
        }
        ret = cardReader.Donsee_ICSelect(type);
        if (ret != 0) {
            listener.onError("TypeA选卡失败 : " + CardReader.getErrorMessage(ret));
            return "-4";
        }
        ret = cardReader.Donsee_PowerOnTypeA(chResponse);
        if (ret > 0) {
            return HexUtil.bytesToHex(chResponse, (int) ret);
        } else {
            listener.onError("TypeA上电失败：" + CardReader.getErrorMessage(ret));
            return "-5";

        }
    }

    /**
     * desc:发送指令
     *
     * @param command      指令  测试  0084000008
     * @param type         1-CPU卡命令，2-PSAM卡命令
     * @param slot         PSAM卡选择卡槽
     * @author xia weihao
     * @date 2024/6/27
     */
    public String sendAPDU(int slot, String command, int type) {
        if (cardReader == null) {
            listener.onError("sendAPDU：cardReader = null");
            return "-1";
        }
        byte[] chResponse = new byte[2048];
        long ret;
        byte cardSlot = 0x11; //SAM1
        if(slot==2){
            cardSlot=0x12;//SAM2
        }
        if (type == 2) {
            ret = cardReader.Donsee_PowerOn_APDU(cardSlot, HexUtil.hexToBytes(command), chResponse);//接触执行apdu
            if (ret > 0) {
                return HexUtil.bytesToHex(chResponse, (int) ret);
            } else {
                listener.onError("接触卡执行APDU失败：" + CardReader.getErrorMessage(ret));
                return "-2";
            }

        } else {
            ret = cardReader.Donsee_APDU(HexUtil.hexToBytes(command), chResponse);//非接执行apdu
            if (ret > 0) {
                return HexUtil.bytesToHex(chResponse, (int) ret);
            } else {
                listener.onError("非接卡执行APDU失败：" + CardReader.getErrorMessage(ret));
                return "-2";
            }
        }
    }

    public String coldReset(int slot){
        if (cardReader == null) {
            listener.onError("coldReset：cardReader = null");
            return "-1";
        }
        byte[] chResponse = new byte[2048];
        long ret;
        byte cardSlot = 0x11; //SAM1
        if(slot==2){
            cardSlot=0x12;//SAM2
        }
        ret = cardReader.Donsee_ColdPowerOn(cardSlot, chResponse);
        if (ret > 0) {
            return HexUtil.bytesToHex(chResponse, (int) ret);
        } else {
            listener.onError("冷复位失败：" + CardReader.getErrorMessage(ret));
            return "-2";
        }
    }

    /**
     * desc:设置蜂鸣时间
     *
     * @param time 设置蜂鸣时间 time 建议填50
     * @author xia weihao
     * @date 2024/6/27
     */
    public boolean startBeep(int time) {
        if (cardReader == null) {
            listener.onError("startBeep：cardReader = null");
            return false;
        }
        long ret = cardReader.Donsee_Beep(time);
        if (ret == 0) {
            return true;
        } else {
            return false;
        }
    }


    public void startRead() {
        if (cReaderDataThread == null) {
            cReaderDataThread = new CReaderThread();
        }
        if (!cReaderDataThread.isAlive()) {
            isRuning = true;
            cReaderDataThread.start();
        }
    }

    public void pauseRead() {
        if (cReaderDataThread != null) {
            cReaderDataThread.pauseThread();
        }
    }

    public void resumeRead() {
        if (cReaderDataThread != null) {
            cReaderDataThread.resumeThread();
        }
    }

    public void stopRead() {
        if (cReaderDataThread == null) {
            return;
        }
        isRuning = false;
        resetSameCardState();
        cReaderDataThread.pause = false;
        cReaderDataThread.interrupt();
        cReaderDataThread = null;
    }


    private void readNFC() {
        byte[] uid = new byte[10];
        byte type = (byte) 0x41; //0x41 表示TypeA/M1卡选卡,  0x43表示TypeB卡选卡
        byte[] chResponse = new byte[2048];
        if (cardReader == null) {
            resetSameCardState();
            listener.onError("readNFC cardReader = null");
            return;
        }
        long ret = cardReader.Donsee_ICRequest();
        if (ret != 0) {
            resetSameCardState();
            listener.onError("TypeA寻卡失败: " + CardReader.getErrorMessage(ret));
            return;
        }
        if (cardReader.Donsee_ICAnticoll(uid) < 0) {
            resetSameCardState();
            listener.onError("TypeA防碰撞失败: " + CardReader.getErrorMessage(ret));
            return;
        }
        ret = cardReader.Donsee_ICSelect(type);
        if (ret != 0) {
            resetSameCardState();
            listener.onError("TypeA选卡失败 : " + CardReader.getErrorMessage(ret));
            return;
        }
        ret = cardReader.Donsee_PowerOnTypeA(chResponse);
        if (ret > 0) {
            String info = HexUtil.bytesToHex(chResponse, (int) ret);
            //todo 同卡在间隔时间内不重复推送逻辑
            long currTime = System.currentTimeMillis();
            long intervalTime = 0L;
            if (config != null && config.getSameCardIntervalTime() > 0) {
                intervalTime = config.getSameCardIntervalTime();
            }
            if (info.equals(oldNFCResetInfo) && currTime > 0) {
                // todo 当同卡间隔时间大于intervalTime，重置字段并发送一次数据
                if (currTime - recordTime > intervalTime) {
                    oldNFCResetInfo = info;
                    recordTime = currTime;
                    listener.onPollingInfo(info);
                }
            } else {
                oldNFCResetInfo = info;
                recordTime = currTime;
                listener.onPollingInfo(info);
            }

        } else {
            resetSameCardState();
            listener.onError("TypeA上电失败：" + CardReader.getErrorMessage(ret));
        }
    }


    /**
     *
     * desc：重置同卡间隔逻辑状态
     * @author xia weihao
     * @date 2024/6/27
     *
     */
    private void resetSameCardState(){
        oldNFCResetInfo = "";
        recordTime = 0;
    }

    /**
     * desc:销毁资源
     *
     * @author xia weihao
     * @date 2024/6/27
     */
    public void release(Context context) {
        stopRead();
        if (cardReader != null) {
            cardReader.Donsee_Close();
            cardReader.unregisterUSBReceiver();
        }

    }


    public interface NFCCardResaultListener {

        void onPollingInfo(String info);

        void onError(String errMsg);


    }

}
